#
#The purpose of this file is to set some sensible defaults when an RServe process starts. 
#The function "masks" are intended to keep the normal user from getting into an undesirable
#situation with interactive prompts that might cause RServe process to block indefinitely.
#They are NOT fool proof
#

init <- function()
{ 
  
  #check if there is a default CRAN mirror specified, so there is not an attempt
  #to "prompt" the user to select one.
  if (pmatch(NA, options("repos")$repos["CRAN"], nomatch=FALSE) || 
        pmatch("@CRAN@", options("repos")$repos["CRAN"], nomatch=FALSE))
  {
    
    #set the default CRAN mirror
    if (!identical(system.file(package="RevoUtils"), "")) 
    {    
      options(repos = c(CRAN = RevoUtils::getRevoRepos()))
    }
    else
    {
      options(repos = c(CRAN = "https://cran.r-project.org/"))
    }
  }
  
  #specify the way we will download files
  if (.Platform$OS.type == "windows") 
  {
    options(download.file.method = "wininet")
  }
  else
  {
    options(download.file.method = "curl")    
  }
  
  
  #functions return NULL
  deployr.help <- function (...) 
  {    
  }
  
  deployr.quit <- function (...) 
  {    
  }
  
  deployr.browser <- function (...) 
  {    
  }
  
  deployr.unlock <- function(...)
  {
  }  
  

  # Support spark cc on HDI
  # On Unix, when RevoScaleR package loads, source R.home('etc')/Rprofile.site
  if (.Platform$OS.type == "unix")
  {
    setHook(packageEvent("RevoScaleR", "onLoad"),
      function(...) {
        cmd<-paste(file.path(R.home("etc")), "/Rprofile.site", sep="")
        capture.output(source(cmd))
      })
  }

  #mask functions in the utils package
  u <- getNamespace("utils")
  unlockBinding("help", u)
  assign("help", deployr.help, envir=u)
  lockBinding("help", u)
  
  #add more here if necessary......
  
  #mask functions in the base package
  b <- getNamespace("base")  
  unlockBinding("q", b)
  assign("q", deployr.quit, envir=b)
  lockBinding("q", b)
  
  unlockBinding("quit", b)
  assign("quit", deployr.quit, envir=b)
  lockBinding("quit", b)
  
  unlockBinding("browser", b)
  assign("browser", deployr.browser, envir=b)
  lockBinding("browser", b)
  
  #add more here if necessary......
}

#some helper functions needed by DeployR
helper<-function()
{
  e <- new.env(parent=globalenv())
  local({
    
    #check to be sure a plot file is at least 400 bytes
    checkUnnamedPlotFileSize<-function(plotNamefilter) 
    {    
      files<-dir(pattern=plotNamefilter)
      for(name in files) 
      {
        info<-file.info(name);       
        if(info$size < 400) 
        {
          unlink(name);
        }
      }  
    }
    
    #install.packages override.  we override for 2 purposes.  First, to make sure that the
    #default location where packages are saved to is the current working directory.  Second,
    #so the UI can show progress of the install.  Otherwise the UI will have to wait until
    #the install is finished.  This could potentially be minutes without any feedback to the
    #user.  That is why we put everything in a script and redirect the output.   Behind the scenes
    #the mrsdeploy package will call /sessions/{id}/console-output" to get the content of 
    #DeployREngineConsole{sesionid}.txt file
    
    install.packages<-function(...)
    {
      #by default we want to install packages to the current working directory
      #so update .libPaths
      .libPaths(getwd())
      
      connName<-NULL
      redirect<-FALSE
    
      #get the connections for the session
      s<-getAllConnections()
      for(i in s)
      {
        #look and see if DeployREngineConsole{sesionid}.txt is open via 'sink' command
        sum<-summary.connection(i)
        if (length(grep("DeployREngineConsole", sum$description)))
        {
          conn<-getConnection(i)
          connName<-sum$description
        }
      }
      
      #remove the sink
      if (!is.null(connName))
      {
        if(sink.number() == 1) 
        {
          sink()
          redirect <-TRUE
        }
      }
      
      #replicate the input parameter passed to 'install.packages'
      plist<-list(...)
      pstr<-deparse(substitute(...))
      
      for (i in 1:length(plist))
      {
        if((!is.null(names(plist[i]))) && (!names(plist[i]) == ""))
        {
          if (is.character(plist[[i]]))
          {
            pstr<-paste(pstr, ", ", names(plist[i]),"=", "\"", plist[i], "\"", sep="")
          }
          else
          {
            pstr<-paste(pstr, ", ", names(plist[i]),"=", plist[i],  sep="")
          }
        }
        else
        {
          pstr<-eval(plist[i])     
          if (length(plist[[i]]) == 1)
          {
            pstr<-paste("\"", pstr, "\"", sep="")
          }
        }
      }
      
      #create a temp file with the commands to execute
      tryCatch({
        code <- 
          paste(
            "environment(.libPaths)$.lib.loc <- ", deparse(.libPaths(), width.cutoff = 500), ";",
            "options(repos = ", deparse(getOption('repos'), width.cutoff = 500), ");",
            "options(configure.vars = ", deparse(getOption('configure.vars'), width.cutoff = 500), ");",
            "options(download.file.method = ", deparse(getOption('download.file.method'), width.cutoff = 500), ");",
            "install.packages(", pstr, ")", sep='')
        
        
        if (redirect == TRUE)
        {
          code<-gsub("\"", "'", code)
          if (.Platform$OS.type == "windows")
          {
            cmd<-paste(file.path(shortPathName(R.home("bin"))), "/Rscript", sep="")
          }
          else
          {
            cmd<-paste(file.path(R.home("bin")), "/Rscript", sep="")
          }
          
          scriptfile <- tempfile()
          on.exit(unlink(scriptfile))
          writeLines(code, scriptfile)
          
          scriptfile<-gsub("\\\\", "/", scriptfile)        
          
          args =c("--quiet", "--vanilla", shQuote(scriptfile), ">>", shQuote(connName), "2>&1")
          
          #execute the command, redirecting the output to the file we removed the sink from
          if(!is.null(connName))
          {
            if (.Platform$OS.type == "windows") 
            {
              out<-system(paste("cmd", "/c", cmd, paste(args, sep="", collapse=" "), sep=" "))
            }
            else
            {
              out<-system2(cmd, args, stdout=TRUE, stderr=TRUE )
            }
          }
        }
      },
      
      #re-establish the sink we removed
      finally = {
        if(!is.null(connName))
        {
          sink(file=connName, append=TRUE)
        }
      }) 
    }
    library<-function(...)
    {
      .libPaths(getwd())
      if (.Platform$OS.type == "unix")
      {
        .libPaths(c(getwd(), "/opt/microsoft/mlserver/9.4.7/libraries/RServer"))
      }
      base::library(...)
    }
    require<-function(...)
    {
      .libPaths(getwd())
      if (.Platform$OS.type == "unix")
      {
        .libPaths(c(getwd(), "/opt/microsoft/mlserver/9.4.7/libraries/RServer"))
      }
      base::require(...)
    }
  }, e)
  #attach the enivronment to a new namespace
  attach(e, name = ".deployr_rserve")
}


#call the init and helper functions
tryCatch({
  init()
  helper()
}, 
warning = function(w) {
  print(w)
}, 
error = function(e) {
  print(e)
}, 
finally = {
  rm(init)
  rm(helper)
})

